from functools import partial

from django.dispatch import receiver
from telethon import events

from django_telethon.signals import telegram_client_registered


async def register_entity(event, client_session):
    # register entity
    await event.get_input_sender()


@receiver(telegram_client_registered)
def receiver_telegram_client_registered(telegram_client, client_session, *args, **kwargs):
    handler = partial(register_entity, client_session=client_session)
    telegram_client.add_event_handler(
        handler,
        events.NewMessage,
    )



from django.conf import settings

from django_telethon.importer import import_attribute


def get_telethon_config(key, default=None):
    """Fetch configuration from DJANGO_TELETHON in Django settings."""
    return getattr(settings, 'DJANGO_TELETHON', {}).get(key, default)


# Retrieve configuration values
RABBITMQ_ACTIVE = get_telethon_config('RABBITMQ_ACTIVE', False)
RABBITMQ_URL = get_telethon_config('RABBITMQ_URL')
QUEUE_CHANNEL_NAME = get_telethon_config('QUEUE_CHANNEL_NAME')
QUEUE_CALLBACK = get_telethon_config('QUEUE_CALLBACK', default='django_telethon.callback.on_message')
QUEUE_CALLBACK_FN = None

# Validate configuration
if RABBITMQ_ACTIVE:
    missing_settings = []

    if not RABBITMQ_URL:
        missing_settings.append('RABBITMQ_URL')
    if not QUEUE_CHANNEL_NAME:
        missing_settings.append('QUEUE_CHANNEL_NAME')

    QUEUE_CALLBACK_FN = import_attribute(QUEUE_CALLBACK)

    if not QUEUE_CALLBACK_FN:
        missing_settings.append('QUEUE_CALLBACK')

    if missing_settings:
        raise ValueError(f"'Telethon' {', '.join(missing_settings)} must be set if RABBITMQ_ACTIVE is True.")





import logging

import aio_pika
import pika

from django_telethon.default_settings import RABBITMQ_URL, QUEUE_CHANNEL_NAME, QUEUE_CALLBACK_FN

_rabbit_registered = False


async def connect_rabbitmq():
    global _rabbit_registered

    if _rabbit_registered:
        return

    _rabbit_registered = True

    connection = await aio_pika.connect_robust(
        RABBITMQ_URL
    )

    # Creating a channel
    channel = await connection.channel()

    # Declare a queue to make sure it exists. If the queue is already there this won't do anything.
    queue = await channel.declare_queue(QUEUE_CHANNEL_NAME, durable=True)

    # Creating a consumer callback
    async def on_message(message: aio_pika.IncomingMessage):
        async with message.process():
            await QUEUE_CALLBACK_FN(message.body)

    await queue.consume(on_message)


def send_to_telegra_thread(**payload):
    try:
        parameters = pika.URLParameters(RABBITMQ_URL)
        connection = pika.BlockingConnection(parameters)
        channel = connection.channel()

        # Ensure the queue exists. You might want to move this outside the function if you know the queue already exists.
        channel.queue_declare(queue=QUEUE_CHANNEL_NAME, durable=True)
        byte_payload = str(payload).encode('utf-8')
        # Send the message
        channel.basic_publish(
            exchange='',
            routing_key=QUEUE_CHANNEL_NAME,
            body=byte_payload,
            properties=pika.BasicProperties(
                delivery_mode=2,  # Make message persistent
            ))

        connection.close()
    except Exception as e:
        logging.error(f"django telethon error occurred: {e}")
        raise e
